#ifndef __FifthES__H
#define __FifthES__H

#include <es.h>
#include <solution.h>

class FifthES : public ES {
public:
  FifthES(int _maxEval,
               double _sigma,
               Eval & _evalf,
               std::mt19937 & _generator) : maxEval(_maxEval), sigma(_sigma), evalf(_evalf), generator(_generator) {
    
  }
  
  virtual void run(Solution & _solution) {
    std::normal_distribution<double> distribution(0.0, 1.0);
    int nbEval = 0;
    
    Solution s2;
    
    while (nbEval < maxEval) {
      _solution.printOn();
      std::cout << std::endl;
      
      s2.x1 = _solution.x1 + sigma * distribution(generator);
      s2.x2 = _solution.x2 + sigma * distribution(generator);
      
      evalf.apply(s2);
      
      if (s2.fitness() < _solution.fitness()) {
        _solution.x1 = s2.x1;
        _solution.x2 = s2.x2;
        _solution.fitness(s2.fitness());
        
        sigma = sigma * exp(1.0 / 3.0);
      } else
        sigma = sigma / pow(exp(1.0 / 3.0), 0.25);
      
      
      nbEval++;
    }
  }

protected:
  int maxEval;
  std::mt19937 & generator;
  double sigma;
  Eval & evalf;
  
};
#endif