/*
 *  To generate sample of points 
 *  following a bi-variate normal distributions
 *
 *  SV : version 1 - 2014/11/12
 *
 * To compile :
 *   g++ -std=c++11 biVariateNormalDistribution.cpp -o biVariateNormalDistribution
 */

#include <iostream>
#include <random>
#include <fstream>

using namespace std;

/*
 Files to produce using different parameters settings:
 
 - out1.dat : sigma1 = 1,  sigma2 = 1,  theta = 0;
 - out2.dat : sigma1 = 10, sigma2 = 10, theta = 0;
 - out3.dat : sigma1 = 1,  sigma2 = 10, theta = 0;
 - out4.dat : sigma1 = 1,  sigma2 = 10, theta = pi/3;

 Plot using gnuplot:
 
   plot "out1.dat" w p, "out2.dat" w p, "out3.dat" w p, "out4.dat" w p

 */

int main()
{
  // mersene-twister pseudo-random number generator
  mt19937 generator;
  // seed of the generator is given by time in millisecond
  generator.seed(time(0));
  // normal distribution generator: mu = 0 and sd = 1
  normal_distribution<double> distribution(0.0, 1.0);
  
  // usefull variables
  double x1, x2;
  double X1, X2;

  double sigma1 = 1;
  double sigma2 = 10;
  double theta = 3.14159 / 3;
  
  // output file
  fstream file("out4.dat", std::ios::out);
 
  // sample 1000 points
  for(int i = 0; i < 1000; i++) {
    // pseudo random numbers following normal distribution
    x1 = sigma1 * distribution(generator);
    x2 = sigma2 * distribution(generator);

    X1 = cos(theta) * x1 - sin(theta) * x2;
    X2 = sin(theta) * x1 + cos(theta) * x2;
    
    file << X1 << " " << X2 << endl;
  }
  
  // close the file
  file.close();
  
  return 0;
}
